@@ -23,18 +23,57 @@ from IrcBot.utils import log, debug, logger
 from IrcBot.sqlitedb import DB
 
 utils = utils
-
 BUFFSIZE = 2048
 
+
+class Color(object):
+    """
+    Colorcodes enum
+    """
+    esc         = '\003'
+    white       = '00'
+    black       = '01'
+    navy        = '02'
+    green       = '03'
+    red         = '04'
+    maroon      = '05'
+    purple      = '06'
+    orange      = '07'
+    yellow      = '08'
+    light_green = '09'
+    teal        = '10'
+    cyan        = '11'
+    blue        = '12'
+    magenta     = '13'
+    gray        = '14'
+    light_gray  = '15'
+
+    COLORS = [
+        '00', '01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12', '13', '14', '15'
+    ]
+
+    def __init__(self, text, fg=white, bg=None):
+        if bg is not None:
+            self.text = "{}{},{}{}".format(self.esc, fg, bg, text)
+        else:
+            self.text = "{}{}{}".format(self.esc, fg, text)
+        self.str = self.text
+
+    def __str__(self):
+        return self.str
+
+
 class dbOperation(object):
     ADD = 1
     UPDATE = 0
     REMOVE = -1
-    def __init__(self, data = {}, id = {}, op = UPDATE):
+
+    def __init__(self, data={}, id={}, op=UPDATE):
         self.data = data
         self.id = id
         self.op = op
 
+
 class tempData(object):
     def __init__(self):
         """Initializes a temporary data object that can be retrieved from the same user and channel.
@@ -61,7 +100,6 @@ class tempData(object):
         return self.data[(msg.channel, msg.sender_nick)]
 
 
-
 class persistentData(object):
     def __init__(self, filename, name, keys):
         """__init__.
@@ -99,12 +137,11 @@ class persistentData(object):
         else:
             self._queue.append(dbOperation(data=items, op=dbOperation.ADD))
 
-
     def pop(self, id):
         """Removes the row based on the id. (You can see with self.data)
         :param id: int
         """
-        assert type(id)==int, "id needs to be an int!"
+        assert type(id) == int, "id needs to be an int!"
         self._queue.append(dbOperation(id=id, op=dbOperation.REMOVE))
 
     def update(self, id, item):
@@ -112,17 +149,17 @@ class persistentData(object):
         :param id: id of item to update, change.
         :param item: New item to replace with. This dict doesn't need to have all keys/columns, just the ones to be changed.
         """
-        assert type(item) == dict , "item must be either list or dict"
-        assert type(id)==int, "id needs to be an int!"
-        self._queue.append(dbOperation(id=id, data=item, op=dbOperation.update))
+        assert type(item) == dict, "item must be either list or dict"
+        assert type(id) == int, "id needs to be an int!"
+        self._queue.append(dbOperation(
+            id=id, data=item, op=dbOperation.update))
 
     def clear(self):
         """Clear all the proposed modifications.
         """
         self._queue = []
-            
 
-        
+
 class Message(object):
     def __init__(self, channel='', sender_nick='', message='', is_private=False):
         self.channel = channel.strip()
@@ -132,6 +169,7 @@ class Message(object):
         self.txt = self.text = self.message
         self.is_private = is_private
 
+
 class ReplyIntent(object):
     def __init__(self, message, func):
         """__init__.
@@ -145,7 +183,7 @@ class ReplyIntent(object):
 class IrcBot(object):
     """IrcBot."""
 
-    def __init__(self, host, port=6665, nick="bot", channels=[], username=None, password='', server_password='', use_sasl=False, use_ssl=False, tables=[]):
+    def __init__(self, host, port=6665, nick="bot", channels=[], username=None, password='', server_password='', use_sasl=False, use_ssl=False, delay=False, tables=[]):
         """Creates a bot instance joining to the channel if specified
 
         :param host: str. Server hostname. ex: "irc.freenode.org"
@@ -169,9 +207,12 @@ class IrcBot(object):
         self.use_sasl = use_sasl
         self.use_ssl = use_ssl
         self.tables = tables
+        self.delay=False
 
-        self.send_message_channel, self.receive_message_channel = trio.open_memory_channel(0)
-        self.send_db_operation_channel, self.receive_db_operation_channel = trio.open_memory_channel(0)
+        self.send_message_channel, self.receive_message_channel = trio.open_memory_channel(
+            0)
+        self.send_db_operation_channel, self.receive_db_operation_channel = trio.open_memory_channel(
+            0)
         self.replyIntents = {}
 
         if not self.username:
@@ -179,25 +220,25 @@ class IrcBot(object):
 
     def run(self):
         trio.run(self.connect)
-    
+
     async def ping_confirmation(self, s):
         MAX = 10
-        c=0
+        c = 0
         log("AWAITING PING CONFIRMATION.....")
         async for data in s:
-            data = data.decode('utf-8') 
+            data = data.decode('utf-8')
             for msg in data.split("\r\n"):
                 debug("RECV --------- " + msg)
-                if c>MAX:
+                if c > MAX:
                     return
-                if "001 "+self.nick+ " :" in msg:
+                if "001 "+self.nick + " :" in msg:
                     return
                 if data.find("PING") != -1 and len(data.split(":")) >= 2:
                     msg = str('PONG :' + data.split(":")[-1])
                     debug("Registration pong: ", msg)
                     await s.send_all(msg.encode())
                     return
-                c+=1
+                c += 1
 
     async def connect(self):
         remote_ip = socket.gethostbyname(self.host)
@@ -215,7 +256,7 @@ class IrcBot(object):
             if self.use_sasl:
                 await s.send_all(("CAP REQ :sasl").encode())
 
-            if self.server_password :
+            if self.server_password:
                 pw_cr = ('PASS ' + self.server_password + '\r\n').encode()
                 await s.send_all(pw_cr)
 
@@ -227,10 +268,12 @@ class IrcBot(object):
             await s.send_all(usernam_cr)
 
             ping_confirmed = False
-            with trio.fail_after(4):
-                await self.ping_confirmation(s)
-                ping_confirmed=True
-                await trio.sleep(2)
+            try:
+                with trio.fail_after(2):
+                    await self.ping_confirmation(s)
+                    ping_confirmed = True
+            except:
+                log("No ping confirmation")
 
             if self.password:
                 log("IDENTIFYING")
@@ -238,14 +281,14 @@ class IrcBot(object):
                            self.password + '\r\n').encode()
                 await s.send_all(auth_cr)
 
-
-            await trio.sleep(2)
+            if self.delay:
+                await trio.sleep(2)
             if self.use_sasl:
                 import base64
                 await s.send_all(("AUTHENTICATE PLAIN").encode())
                 sep = "\x00"
                 b = base64.b64encode(
-                    (nick + sep+nick+sep+password).encode("utf8")).decode("utf8")
+                    (self.nick + sep+self.nick+sep+self.password).encode("utf8")).decode("utf8")
                 data = s.recv(4096).decode('utf-8')
                 log("Server SAYS: ", data)
                 await s.send_all(("AUTHENTICATE "+b).encode())
@@ -256,7 +299,6 @@ class IrcBot(object):
                 log("Server SAYS: ", data)
                 await s.send_all(("CAP END").encode())
 
-
             self.s = s
             if type(self.channels) == list:
                 for c in self.channels:
@@ -290,11 +332,13 @@ class IrcBot(object):
             for chan in channel:
                 await self._send_message(message, chan)
         else:
-           await self._send_message(message, channel)
+            await self._send_message(message, channel)
 
     async def _send_message(self, message, channel):
         if type(message) == str:
             await self._enqueue_message((str('PRIVMSG ' + channel) + " :" + message + ' \r\n'))
+        elif type(message) == Color:
+            await self._send_message(message.str, channel)
         elif type(message) == list:
             for msg in message:
                 await self._send_message(msg, channel)
@@ -319,7 +363,8 @@ class IrcBot(object):
         debug("Checking tables")
         for table in self.tables:
             if table._queue:
-                table_copy = persistentData(table.filename, table.name, table.keys)
+                table_copy = persistentData(
+                    table.filename, table.name, table.keys)
                 table_copy._queue = table._queue
                 await self._enqueue_db_tsk(table_copy)
             debug("qeue", table._queue)
@@ -345,14 +390,14 @@ class IrcBot(object):
                     if op.op == dbOperation.UPDATE:
                         table.db.update(op.id, op.data)
 
-            
     async def run_bot_loop(self, s):
         """ Starts main bot loop waiting for messages.
         """
         async with self.send_message_channel, self.send_db_operation_channel:
             async for data in s:
-                data = data.decode('utf-8') 
-                debug("DECODED DATA FROM SERVER: \n", 40*"-", "\n", data, 40*"-", "\n")
+                data = data.decode('utf-8')
+                debug("DECODED DATA FROM SERVER: \n",
+                      40*"-", "\n", data, 40*"-", "\n")
                 async with trio.open_nursery() as nursery:
                     self.fetch_tables()
                     for msg in data.split("\r\n"):
@@ -420,16 +465,17 @@ class IrcBot(object):
                 matched = False
 
                 if channel in self.replyIntents and sender_nick in self.replyIntents[channel]:
-                    result = self.replyIntents[channel][sender_nick].func(Message(channel, sender_nick, msg, is_private))
+                    result = self.replyIntents[channel][sender_nick].func(
+                        Message(channel, sender_nick, msg, is_private))
                     del self.replyIntents[channel][sender_nick]
                     await self.process_result(result, channel, sender_nick, is_private)
                     return
 
-                for i, cmd in enumerate(utils.regex_commands): 
+                for i, cmd in enumerate(utils.regex_commands):
                     if matched:
                         break
                     for reg in cmd:
-                        m = re.match(reg, msg) #, flags=re.IGNORECASE)
+                        m = re.match(reg, msg)  # , flags=re.IGNORECASE)
                         if m:
                             if cmd in utils.regex_commands:
                                 if is_private and not utils.regex_commands_accept_pm[i]:
@@ -445,7 +491,7 @@ class IrcBot(object):
                     if matched:
                         break
                     for reg in cmd:
-                        m = re.match(reg, msg) #, flags=re.IGNORECASE)
+                        m = re.match(reg, msg)  # , flags=re.IGNORECASE)
                         if m:
                             if cmd in utils.regex_commands_with_message:
                                 if is_private and not utils.regex_commands_with_message_accept_pm[i]:
@@ -453,10 +499,12 @@ class IrcBot(object):
                                 debug("sending to", sender_nick)
                                 if utils.regex_commands_with_message_pass_data[i]:
                                     await trio.sleep(0)
-                                    result = cmd[reg](m, Message(channel, sender_nick, msg, is_private))
+                                    result = cmd[reg](m, Message(
+                                        channel, sender_nick, msg, is_private))
                                 else:
                                     await trio.sleep(0)
-                                    result = cmd[reg](m, Message(channel, sender_nick, msg, is_private))
+                                    result = cmd[reg](m, Message(
+                                        channel, sender_nick, msg, is_private))
 
                             if result:
                                 await self.process_result(result, channel, sender_nick, is_private)
@@ -492,4 +540,3 @@ class IrcBot(object):
     def close(self):
         """Stops the bot and loop if running."""
         self.__del__()
-
